import { inject, Injectable } from '@angular/core';
import { Subject } from 'rxjs';
import { AuthConfig, OAuthService } from 'angular-oauth2-oidc';
import { UserService } from './user.service';
import { User } from '../model/User';


@Injectable({
  providedIn: 'root',
})
export class AuthService {
  private userService: UserService = inject(UserService);

  private readonly authConfig: AuthConfig = {
    issuer: 'https://oauth.simonis.lol/application/o/casino-dev/',
    loginUrl: 'https://oauth.simonis.lol/application/o/authorize/',
    tokenEndpoint: 'https://oauth.simonis.lol/application/o/token/',
    userinfoEndpoint: 'https://oauth.simonis.lol/application/o/userinfo/',
    clientId: 'MDqjm1kcWKuZfqHJXjxwAV20i44aT7m4VhhTL3Nm',
    dummyClientSecret: 'GY2F8te6iAVYt1TNAUVLzWZEXb6JoMNp6chbjqaXNq4gS5xTDL54HqBiAlV1jFKarN28LQ7FUsYX4SbwjfEhZhgeoKuBnZKjR9eiu7RawnGgxIK9ffvUfMkjRxnmiGI5',
    scope: 'openid profile email',
    responseType: 'code',
    redirectUri: window.location.origin + '/auth/callback',
    oidc: true,
    requestAccessToken: true,
    strictDiscoveryDocumentValidation: false,
    showDebugInformation: true,
  };

  private isAuthenticated = new Subject<boolean>();
  private user: User | null = null;
  private oauthService: OAuthService = inject(OAuthService);

  constructor() {
    this.oauthService.configure(this.authConfig);
    this.oauthService.events.subscribe((event) => {
      if (event.type === 'token_received') {
        this.oauthService.loadUserProfile().then((profile) => {
          this.fromUserProfile(profile).subscribe((user) => {
            this.user = user;
          });
        });
      }
    });
    this.oauthService.loadDiscoveryDocumentAndTryLogin().then(() => {
      this.isAuthenticated.next(this.oauthService.hasValidAccessToken());
    });
  }

  login() {
    this.oauthService.initLoginFlow();
  }

  logout() {
    this.oauthService.logOut();
    this.isAuthenticated.next(false);
  }

  getUserInfo() {
    return this.user;
  }

  isLoggedIn() {
    return this.oauthService.hasValidAccessToken();
  }

  private fromUserProfile(profile: object) {
    return this.userService.getOrCreateUser(profile);
  }

  getAccessToken() {
    return this.oauthService.getAccessToken();
  }
}
